@@ -3,10 +3,10 @@ from collections import OrderedDict
 from _pytest.runner import CollectReport
 from py.log import Producer
 from xdist.report import report_collection_diff
-from xdist.slavemanage import parse_spec_config
+from xdist.workermanage import parse_spec_config
 
 
-class LoadScopeScheduling:
+class LoadScopeScheduling(object):
     """Implement load scheduling across nodes, but grouping test by scope.
 
     This distributes the tests collected across all nodes so each test is run
@@ -93,7 +93,7 @@ class LoadScopeScheduling:
         self.registered_collections = OrderedDict()
 
         if log is None:
-            self.log = Producer('loadscopesched')
+            self.log = Producer("loadscopesched")
         else:
             self.log = log.loadscopesched
 
@@ -151,7 +151,7 @@ class LoadScopeScheduling:
 
         From now on the node will be assigned work units to be executed.
 
-        Called by the ``DSession.slave_slaveready`` hook when it successfully
+        Called by the ``DSession.worker_workerready`` hook when it successfully
         bootstraps a new node.
         """
         assert node not in self.assigned_work
@@ -166,8 +166,8 @@ class LoadScopeScheduling:
 
         Called by the hooks:
 
-        - ``DSession.slave_slavefinished``.
-        - ``DSession.slave_errordown``.
+        - ``DSession.worker_workerfinished``.
+        - ``DSession.worker_errordown``.
 
         Return the item being executed while the node crashed or None if the
         node has no more pending items.
@@ -187,8 +187,7 @@ class LoadScopeScheduling:
             break
         else:
             raise RuntimeError(
-                'Unable to identify crashitem on a workload with '
-                'pending items'
+                "Unable to identify crashitem on a workload with " "pending items"
             )
 
         # Made uncompleted work unit available again
@@ -206,7 +205,7 @@ class LoadScopeScheduling:
 
         Called by the hook:
 
-        - ``DSession.slave_collectionfinish``.
+        - ``DSession.worker_collectionfinish``.
         """
 
         # Check that add_node() was called on the node before
@@ -224,10 +223,7 @@ class LoadScopeScheduling:
                 other_node = next(iter(self.registered_collections.keys()))
 
                 msg = report_collection_diff(
-                    self.collection,
-                    collection,
-                    other_node.gateway.id,
-                    node.gateway.id
+                    self.collection, collection, other_node.gateway.id, node.gateway.id
                 )
                 self.log(msg)
                 return
@@ -239,7 +235,7 @@ class LoadScopeScheduling:
 
         Called by the hook:
 
-        - ``DSession.slave_testreport``.
+        - ``DSession.worker_testreport``.
         """
         nodeid = self.registered_collections[node][item_index]
         scope = self._split_scope(nodeid)
@@ -255,9 +251,7 @@ class LoadScopeScheduling:
         scope, work_unit = self.workqueue.popitem(last=False)
 
         # Keep track of the assigned work
-        assigned_to_node = self.assigned_work.setdefault(
-            node, default=OrderedDict()
-        )
+        assigned_to_node = self.assigned_work.setdefault(node, default=OrderedDict())
         assigned_to_node[scope] = work_unit
 
         # Ask the node to execute the workload
@@ -292,14 +286,11 @@ class LoadScopeScheduling:
             example/loadsuite/test/test_delta.py::Delta1
             example/loadsuite/epsilon/__init__.py
         """
-        return nodeid.rsplit('::', 1)[0]
+        return nodeid.rsplit("::", 1)[0]
 
     def _pending_of(self, workload):
         """Return the number of pending tests in a workload."""
-        pending = sum(
-            list(scope.values()).count(False)
-            for scope in workload.values()
-        )
+        pending = sum(list(scope.values()).count(False) for scope in workload.values())
         return pending
 
     def _reschedule(self, node):
@@ -317,7 +308,7 @@ class LoadScopeScheduling:
         if not self.workqueue:
             return
 
-        self.log('Number of units waiting for node:', len(self.workqueue))
+        self.log("Number of units waiting for node:", len(self.workqueue))
 
         # Check that the node is almost depleted of work
         # 2: Heuristic of minimum tests to enqueue more work
@@ -336,7 +327,7 @@ class LoadScopeScheduling:
 
         If ``.collection_is_completed`` is True, this is called by the hook:
 
-        - ``DSession.slave_collectionfinish``.
+        - ``DSession.worker_collectionfinish``.
         """
         assert self.collection_is_completed
 
@@ -348,13 +339,11 @@ class LoadScopeScheduling:
 
         # Check that all nodes collected the same tests
         if not self._check_nodes_have_same_collection():
-            self.log('**Different tests collected, aborting run**')
+            self.log("**Different tests collected, aborting run**")
             return
 
         # Collections are identical, create the final list of items
-        self.collection = list(
-            next(iter(self.registered_collections.values()))
-        )
+        self.collection = list(next(iter(self.registered_collections.values())))
         if not self.collection:
             return
 
@@ -368,18 +357,22 @@ class LoadScopeScheduling:
         extra_nodes = len(self.nodes) - len(self.workqueue)
 
         if extra_nodes > 0:
-            self.log('Shuting down {0} nodes'.format(extra_nodes))
+            self.log("Shuting down {0} nodes".format(extra_nodes))
 
             for _ in range(extra_nodes):
                 unused_node, assigned = self.assigned_work.popitem(last=True)
 
-                self.log('Shuting down unused node {0}'.format(unused_node))
+                self.log("Shuting down unused node {0}".format(unused_node))
                 unused_node.shutdown()
 
         # Assign initial workload
         for node in self.nodes:
             self._assign_work_unit(node)
 
+        # Ensure nodes start with at least two work units if possible (#277)
+        for node in self.nodes:
+            self._reschedule(node)
+
         # Initial distribution sent all tests, start node shutdown
         if not self.workqueue:
             for node in self.nodes:
@@ -398,10 +391,7 @@ class LoadScopeScheduling:
 
         for node, collection in node_collection_items[1:]:
             msg = report_collection_diff(
-                col,
-                collection,
-                first_node.gateway.id,
-                node.gateway.id,
+                col, collection, first_node.gateway.id, node.gateway.id
             )
             if not msg:
                 continue
@@ -412,12 +402,7 @@ class LoadScopeScheduling:
             if self.config is None:
                 continue
 
-            rep = CollectReport(
-                node.gateway.id,
-                'failed',
-                longrepr=msg,
-                result=[]
-            )
+            rep = CollectReport(node.gateway.id, "failed", longrepr=msg, result=[])
             self.config.hook.pytest_collectreport(report=rep)
 
         return same_collection
